# -*- coding: utf-8 -*-
import pytest

from questionary import Separator
from tests.utils import feed_cli_with_input, KeyInputs


def test_submit_empty():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == []


def test_select_first_choice():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = KeyInputs.SPACE + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == ["foo"]


def test_select_and_deselct():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = (KeyInputs.SPACE + KeyInputs.DOWN + KeyInputs.SPACE +
            KeyInputs.SPACE + KeyInputs.ENTER + "\r")

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == ["foo"]


def test_select_first_and_third_choice():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = (KeyInputs.SPACE + KeyInputs.DOWN + KeyInputs.SPACE +
            KeyInputs.DOWN + KeyInputs.ENTER + "\r")

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == ["foo", "bar"]


def test_cycle_to_first_choice():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = (KeyInputs.DOWN + KeyInputs.DOWN +
            KeyInputs.DOWN + KeyInputs.SPACE + KeyInputs.ENTER + "\r")

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == ["foo"]


def test_cycle_backwards():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = KeyInputs.UP + KeyInputs.SPACE + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == ["bazz"]


def test_separator_down():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', Separator(), 'bazz']
    }
    text = KeyInputs.DOWN + KeyInputs.SPACE + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == ["bazz"]


def test_separator_up():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', Separator(), 'bazz', Separator("--END--")]
    }
    text = (KeyInputs.UP + KeyInputs.UP + KeyInputs.SPACE +
            KeyInputs.ENTER + "\r")

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == ["foo"]


def test_select_all():
    message = 'Foo message'
    kwargs = {
        'choices': [
            {"name": "foo", "checked": True},
            Separator(),
            {"name": "bar", "disabled": "nope"},
            'bazz',
            Separator("--END--")]
    }
    text = KeyInputs.UP + "a" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == ["foo", "bazz"]


def test_select_all_deselect():
    message = 'Foo message'
    kwargs = {
        'choices': [
            {"name": "foo", "checked": True},
            Separator(),
            {"name": "bar", "disabled": "nope"},
            'bazz',
            Separator("--END--")]
    }
    text = KeyInputs.UP + "a" + "a" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == []


def test_select_invert():
    message = 'Foo message'
    kwargs = {
        'choices': [
            {"name": "foo", "checked": True},
            Separator(),
            {"name": "bar", "disabled": "nope"},
            'bazz',
            Separator("--END--")]
    }
    text = KeyInputs.UP + "i" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == ["bazz"]


def test_list_random_input():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bazz']
    }
    text = "sdf" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('checkbox', message, text, **kwargs)
    assert result == []


def test_list_ctr_c():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bazz']
    }
    text = KeyInputs.CONTROLC

    with pytest.raises(KeyboardInterrupt):
        feed_cli_with_input('checkbox', message, text, **kwargs)



# -*- coding: utf-8 -*-
import inspect

from prompt_toolkit.document import Document
from prompt_toolkit.shortcuts.prompt import (
    PromptSession)
from prompt_toolkit.styles import merge_styles
from prompt_toolkit.validation import Validator, ValidationError

from questionary.constants import DEFAULT_STYLE


def question(message,
             qmark="?",
             default="",
             validate=None,
             style=None,
             **kwargs):
    merged_style = merge_styles([DEFAULT_STYLE, style])
    validator = None

    if validate:
        if inspect.isclass(validate) and issubclass(validate, Validator):
            validator = validate()
        elif callable(validate):
            class _InputValidator(Validator):
                def validate(self, document):
                    verdict = validate(document.text)
                    if verdict is not True:
                        if verdict is False:
                            verdict = 'invalid input'
                        raise ValidationError(
                            message=verdict,
                            cursor_position=len(document.text))

            validator = _InputValidator()

    def get_prompt_tokens():
        return [("class:qmark", qmark),
                ("class:question", ' {} '.format(message))]

    p = PromptSession(get_prompt_tokens,
                      style=merged_style,
                      validator=validator,
                      **kwargs)
    p.default_buffer.reset(Document(default))

    return p.app



# -*- coding: utf-8 -*-

from prompt_toolkit.output import ColorDepth

from questionary import prompts, utils
import prompt_toolkit.patch_stdout


class PromptParameterException(ValueError):
    def __init__(self, message, errors=None):
        # Call the base class constructor with the parameters it needs
        super(PromptParameterException, self).__init__(
            'You must provide a `%s` value' % message, errors)


def prompt(questions,
           answers=None,
           patch_stdout=False,
           true_color=False,
           kbi_msg='Cancelled by user',
           **kwargs):
    if isinstance(questions, dict):
        questions = [questions]

    answers = answers or {}

    for question in questions:
        # import the question
        if 'type' not in question:
            raise PromptParameterException('type')
        if 'name' not in question:
            raise PromptParameterException('name')

        choices = question.get('choices')
        if choices is not None and callable(choices):
            question['choices'] = choices(answers)

        _kwargs = {}
        _kwargs.update(kwargs)
        _kwargs.update(question)
        _type = _kwargs.pop('type')
        name = _kwargs.pop('name')
        when = _kwargs.pop('when', None)
        _filter = _kwargs.pop('filter', None)

        if true_color:
            _kwargs["color_depth"] = ColorDepth.TRUE_COLOR

        try:
            if when:
                # at least a little sanity check!
                if callable(question['when']):
                    try:
                        if not question['when'](answers):
                            continue
                    except Exception as e:
                        raise ValueError("Problem in 'when' check of {} "
                                         "question: {}".format(name, e))
                else:
                    raise ValueError("'when' needs to be function that "
                                     "accepts a dict argument")
            if _filter:
                # at least a little sanity check!
                if not callable(_filter):
                    raise ValueError("'filter' needs to be function that "
                                     "accepts an argument")

            if callable(question.get('default')):
                _kwargs['default'] = question['default'](answers)

            question_f = getattr(prompts, _type).question

            missing_args = list(utils.missing_arguments(question_f, _kwargs))
            if missing_args:
                raise PromptParameterException(missing_args[0])

            application = question_f(**_kwargs)

            if patch_stdout:
                with prompt_toolkit.patch_stdout.patch_stdout():
                    answer = application.run()
            else:
                answer = application.run()

            if answer is not None:
                if _filter:
                    try:
                        answer = _filter(answer)
                    except Exception as e:
                        raise ValueError("Problem processing 'filter' of {} "
                                         "question: {}".format(name, e))
                answers[name] = answer
        except AttributeError as e:
            print(e)
            raise ValueError("No question type '{}'".format(_type))
        except KeyboardInterrupt:
            print('')
            print(kbi_msg)
            print('')
            return {}
    return answers



# -*- coding: utf-8 -*-
import inspect


def default_values_of(func):
    """Return the defaults of the function `func`."""

    signature = inspect.signature(func)
    return [k
            for k, v in signature.parameters.items()
            if v.default is not inspect.Parameter.empty or
            v.kind != inspect.Parameter.POSITIONAL_OR_KEYWORD]


def arguments_of(func):
    """Return the parameters of the function `func`."""

    return list(inspect.signature(func).parameters.keys())


def required_arguments(func):
    """Return all arguments of a function that do not have a default value."""
    defaults = default_values_of(func)
    args = arguments_of(func)

    if defaults:
        args = args[:-len(defaults)]
    return args  # all args without default values


def missing_arguments(func, argdict):
    """Return all arguments that are missing to call func."""
    return set(required_arguments(func)) - set(argdict.keys())



# -*- coding: utf-8 -*-
from prompt_toolkit import PromptSession
from prompt_toolkit.formatted_text import (
    to_formatted_text)
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.keys import Keys
from prompt_toolkit.styles import merge_styles

from questionary.constants import NO_OR_YES, YES, NO, YES_OR_NO, DEFAULT_STYLE


def question(message,
             qmark="?",
             default=True,
             style=None,
             **kwargs):
    """Create a `PromptSession` object for the 'confirm' function."""

    merged_style = merge_styles([DEFAULT_STYLE, style])

    status = {'answer': None}

    def get_prompt_tokens():
        tokens = []

        tokens.append(("class:qmark", qmark))
        tokens.append(("class:question", ' {} '.format(message)))

        if status['answer'] is not None:
            answer = ' {}'.format(YES if status['answer'] else NO)
            tokens.append(("class:answer", answer))
        else:
            instruction = ' {}'.format(YES_OR_NO if default else NO_OR_YES)
            tokens.append(("class:instruction", instruction))

        return to_formatted_text(tokens)

    bindings = KeyBindings()

    @bindings.add(Keys.ControlQ, eager=True)
    @bindings.add(Keys.ControlC, eager=True)
    def _(event):
        event.app.exit(exception=KeyboardInterrupt, style='class:aborting')

    @bindings.add('n')
    @bindings.add('N')
    def key_n(event):
        status['answer'] = False
        event.app.exit(result=False)

    @bindings.add('y')
    @bindings.add('Y')
    def key_y(event):
        status['answer'] = True
        event.app.exit(result=True)

    @bindings.add(Keys.ControlM, eager=True)
    def set_answer(event):
        status['answer'] = default
        event.app.exit(result=default)

    @bindings.add(Keys.Any)
    def other(event):
        """Disallow inserting other text."""
        pass

    return PromptSession(get_prompt_tokens,
                         key_bindings=bindings,
                         style=merged_style,
                         **kwargs).app



# -*- coding: utf-8 -*-

from prompt_toolkit.application import Application
from prompt_toolkit.filters import IsDone
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.keys import Keys
from prompt_toolkit.layout import Layout
from prompt_toolkit.layout.containers import (
    ConditionalContainer,
    HSplit)
from prompt_toolkit.layout.containers import Window
from prompt_toolkit.shortcuts.prompt import (
    PromptSession)
from prompt_toolkit.styles import merge_styles

from questionary.constants import DEFAULT_STYLE
from questionary.prompts.common import InquirerControl


def question(message,
             choices,
             default=None,
             qmark="?",
             style=None,
             **kwargs):
    merged_style = merge_styles([DEFAULT_STYLE, style])

    ic = InquirerControl(choices, default, use_indicator=False)

    def get_prompt_tokens():
        tokens = []

        tokens.append(("class:qmark", qmark))
        tokens.append(("class:question", ' {} '.format(message)))
        if ic.answered:
            tokens.append(("class:answer", ' ' + ic.get_pointed_at().title))
        else:
            tokens.append(("class:instruction", ' (Use arrow keys)'))

        return tokens

    ps = PromptSession(get_prompt_tokens, reserve_space_for_menu=0, **kwargs)

    layout = Layout(HSplit([
        ps.layout.container,
        ConditionalContainer(
            Window(ic),
            filter=~IsDone()
        )
    ]))

    bindings = KeyBindings()

    @bindings.add(Keys.ControlQ, eager=True)
    @bindings.add(Keys.ControlC, eager=True)
    def _(event):
        event.app.exit(exception=KeyboardInterrupt, style='class:aborting')

    @bindings.add(Keys.Down, eager=True)
    def move_cursor_down(event):
        ic.select_next()
        while not ic.is_selection_valid():
            ic.select_next()

    @bindings.add(Keys.Up, eager=True)
    def move_cursor_up(event):
        ic.select_previous()
        while not ic.is_selection_valid():
            ic.select_previous()

    @bindings.add(Keys.ControlM, eager=True)
    def set_answer(event):
        ic.answered = True
        event.app.exit(result=ic.get_pointed_at().value)

    @bindings.add(Keys.Any)
    def other(event):
        """Disallow inserting other text. """
        pass

    return Application(
        layout=layout,
        key_bindings=bindings,
        style=merged_style,
        **kwargs
    )



# -*- coding: utf-8 -*-
from prompt_toolkit.layout import FormattedTextControl
from prompt_toolkit.validation import Validator, ValidationError

from questionary.constants import (SELECTED_POINTER, INDICATOR_SELECTED,
                                   INDICATOR_UNSELECTED)


class Choice(object):

    def __init__(self, title, value, disabled):
        self.disabled = disabled
        self.value = value
        self.title = title


class Separator(Choice):
    """Used to space/separate choices group"""

    line = '-' * 15

    def __init__(self, line=None):
        if line:
            self.line = line
        super(Separator, self).__init__(self.line, None, True)


class InquirerControl(FormattedTextControl):
    def __init__(self, choices, default, use_indicator=True, **kwargs):
        self.selected_option_index = 0
        self.answered = False
        self.use_indicator = use_indicator
        self.default = default
        self.choices = []  # type: List[Choice]
        self.selected_options = []  # list of names
        self._init_choices(choices, default)
        super(InquirerControl, self).__init__(self._get_choice_tokens,
                                              **kwargs)

    def _is_selected(self, checked, choice):
        return ((checked or choice.value == self.default and
                 self.default is not None) and
                not choice.disabled)

    def _init_choices(self, choices, default=None):
        # helper to convert from question format to internal format
        self.choices = []  # list (name, value, disabled)
        searching_first_choice = True
        for i, c in enumerate(choices):
            if isinstance(c, Separator):
                choice = c
            else:
                if isinstance(c, str):
                    choice = Choice(c, c, False)
                else:
                    choice = Choice(c.get('name'),
                                    c.get('value', c.get('name')),
                                    c.get('disabled', None))
                    if self._is_selected(c.get('checked'), choice):
                        self.selected_options.append(choice.value)

                if searching_first_choice and not choice.disabled:
                    # find the first (available) choice
                    self.selected_option_index = i
                    searching_first_choice = False

            self.choices.append(choice)

    @property
    def choice_count(self):
        return len(self.choices)

    def _get_choice_tokens(self):
        tokens = []

        def append(index, choice):
            # use value to check if option has been selected
            selected = (choice.value in self.selected_options)
            pointed_at = (index == self.selected_option_index)

            if pointed_at:
                tokens.append(("class:pointer",
                               ' {} '.format(SELECTED_POINTER)))
                tokens.append(('[SetCursorPosition]', ''))
            else:
                tokens.append(("", '   '))

            if isinstance(choice, Separator):
                tokens.append(("class:separator",
                               "{}".format(choice.title)))
            elif choice.disabled:  # disabled
                tokens.append(("class:selected" if selected else "",
                               '- {} ({})'.format(choice.title,
                                                  choice.disabled)))
            else:
                if selected:
                    if self.use_indicator:
                        indicator = INDICATOR_SELECTED + " "
                    else:
                        indicator = ""
                    tokens.append(("class:selected",
                                   "{}{}".format(indicator, choice.title)))
                else:
                    if self.use_indicator:
                        indicator = INDICATOR_UNSELECTED + " "
                    else:
                        indicator = ""

                    tokens.append(("",
                                   "{}{}".format(indicator, choice.title)))

            tokens.append(("", '\n'))

        # prepare the select choices
        for i, choice in enumerate(self.choices):
            append(i, choice)
        tokens.pop()  # Remove last newline.
        return tokens

    def is_selection_a_separator(self):
        selected = self.choices[self.selected_option_index]
        return isinstance(selected, Separator)

    def is_selection_disabled(self):
        return self.choices[self.selected_option_index].disabled

    def is_selection_valid(self):
        return (not self.is_selection_disabled() and
                not self.is_selection_a_separator())

    def select_previous(self):
        self.selected_option_index = (
                (self.selected_option_index - 1) % self.choice_count)

    def select_next(self):
        self.selected_option_index = (
                (self.selected_option_index + 1) % self.choice_count)

    def get_pointed_at(self):
        return self.choices[self.selected_option_index]

    def get_selected_values(self):
        # get values not labels
        return [c
                for c in self.choices
                if (not isinstance(c, Separator) and
                    c.value in self.selected_options)]



# -*- coding: utf-8 -*-
import pytest

from tests.utils import feed_cli_with_input, KeyInputs


def test_confirm_enter_default_yes():
    message = 'Foo message'
    text = KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('confirm', message, text)
    assert result is True


def test_confirm_enter_default_no():
    message = 'Foo message'
    text = KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('confirm', message, text, default=False)
    assert result is False


def test_confirm_yes():
    message = 'Foo message'
    text = "y" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('confirm', message, text)
    assert result is True


def test_confirm_no():
    message = 'Foo message'
    text = "n" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('confirm', message, text)
    assert result is False


def test_confirm_big_yes():
    message = 'Foo message'
    text = "Y" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('confirm', message, text)
    assert result is True


def test_confirm_big_no():
    message = 'Foo message'
    text = "N" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('confirm', message, text)
    assert result is False


def test_confirm_random_input():
    message = 'Foo message'
    text = "my stuff" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('confirm', message, text)
    assert result is True


def test_confirm_ctr_c():
    message = 'Foo message'
    text = KeyInputs.CONTROLC

    with pytest.raises(KeyboardInterrupt):
        feed_cli_with_input('confirm', message, text)



# -*- coding: utf-8 -*-
import pytest

from questionary import Separator
from tests.utils import feed_cli_with_input, KeyInputs


def test_select_first_choice():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('list', message, text, **kwargs)
    assert result == 'foo'


def test_select_second_choice():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = KeyInputs.DOWN + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('list', message, text, **kwargs)
    assert result == 'bar'


def test_select_third_choice():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = KeyInputs.DOWN + KeyInputs.DOWN + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('list', message, text, **kwargs)
    assert result == 'bazz'


def test_cycle_to_first_choice():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = (KeyInputs.DOWN + KeyInputs.DOWN +
            KeyInputs.DOWN + KeyInputs.ENTER + "\r")

    result, cli = feed_cli_with_input('list', message, text, **kwargs)
    assert result == 'foo'


def test_cycle_backwards():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bar', 'bazz']
    }
    text = KeyInputs.UP + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('list', message, text, **kwargs)
    assert result == 'bazz'


def test_separator_down():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', Separator(), 'bazz']
    }
    text = KeyInputs.DOWN + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('list', message, text, **kwargs)
    assert result == 'bazz'


def test_separator_up():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', Separator(), 'bazz', Separator("--END--")]
    }
    text = KeyInputs.UP + KeyInputs.UP + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('list', message, text, **kwargs)
    assert result == 'foo'


def test_list_random_input():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bazz']
    }
    text = "some random input" + KeyInputs.ENTER + "\r"

    result, cli = feed_cli_with_input('list', message, text, **kwargs)
    assert result == 'foo'


def test_list_ctr_c():
    message = 'Foo message'
    kwargs = {
        'choices': ['foo', 'bazz']
    }
    text = KeyInputs.CONTROLC

    with pytest.raises(KeyboardInterrupt):
        feed_cli_with_input('list', message, text, **kwargs)



# noinspection PyUnresolvedReferences
from prompt_toolkit.validation import Validator, ValidationError

import questionary.version
from questionary.prompt import prompt
from questionary.prompts.common import Separator

__version__ = questionary.version.__version__



# -*- coding: utf-8 -*-
from tests.utils import feed_cli_with_input


def test_password_entry():
    message = 'What is your password'
    text = "my password\r"

    result, cli = feed_cli_with_input('password', message, text)
    assert result == 'my password'



# -*- coding: utf-8 -*-

from prompt_toolkit.styles import Style

YES = "Yes"

NO = "No"

YES_OR_NO = "(Y/n)"

NO_OR_YES = "(y/N)"

SELECTED_POINTER = "»"

INDICATOR_SELECTED = "●"

INDICATOR_UNSELECTED = "○"

DEFAULT_STYLE = Style([
    ('qmark', 'fg:#5f819d'),
    ('question', 'bold'),
    ('answer', 'fg:#FF9D00 bold'),
    ('pointer', ''),
    ('selected', ''),
    ('separator', ''),
])



from setuptools import setup, find_packages
import io
import os

here = os.path.abspath(os.path.dirname(__file__))

# Avoids IDE errors, but actual version is read from version.py
__version__ = None
exec(open("questionary/version.py").read())

# Get the long description from the README file
with io.open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

tests_requires = [
    "pytest~=4.0",
    "pytest-pep8~=1.0",
    "pytest-cov~=2.6",
    "coveralls~=1.3"
]

install_requires = [
    "prompt_toolkit~=2.0"
]

extras_requires = {
    "test": tests_requires
}

setup(
    name="questionary",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        # supported python versions
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries",
    ],
    packages=find_packages(exclude=["tests", "examples"]),
    version=__version__,
    install_requires=install_requires,
    tests_require=tests_requires,
    extras_require=extras_requires,
    include_package_data=True,
    description="Python library to build pretty command line user prompts ⭐️",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Tom Bocklisch",
    author_email="tombocklisch@gmail.com",
    maintainer="Tom Bocklisch",
    maintainer_email="tombocklisch@gmail.com",
    license="MIT",
    keywords="cli ui inquirer questions prompt",
    url="https://github.com/tmbo/questionary",
    download_url="https://github.com/tmbo/questionary/archive/{}.tar.gz"
                 "".format(__version__),
    project_urls={
        "Bug Reports": "https://github.com/tmbo/questionary/issues",
        "Source": "https://github.com/tmbo/questionary",
    },
)



# -*- coding: utf-8 -*-

from prompt_toolkit.application import Application
from prompt_toolkit.filters import IsDone
from prompt_toolkit.key_binding import KeyBindings
from prompt_toolkit.keys import Keys
from prompt_toolkit.layout import Layout
from prompt_toolkit.layout.containers import (
    ConditionalContainer,
    HSplit)
from prompt_toolkit.layout.containers import Window
from prompt_toolkit.shortcuts.prompt import (
    PromptSession)
from prompt_toolkit.styles import merge_styles

from questionary.constants import DEFAULT_STYLE
from questionary.prompts.common import Separator, InquirerControl


def question(message,
             choices,
             default=None,
             qmark="?",
             style=None,
             **kwargs):
    merged_style = merge_styles([DEFAULT_STYLE, style])

    ic = InquirerControl(choices, default)

    def get_prompt_tokens():
        tokens = []

        tokens.append(("class:qmark", qmark))
        tokens.append(("class:question", ' {} '.format(message)))
        if ic.answered:
            nbr_selected = len(ic.selected_options)
            if nbr_selected == 0:
                tokens.append(("class:answer", ' done'))
            elif nbr_selected == 1:
                tokens.append(("class:answer",
                               ' [{}]'.format(
                                   ic.get_selected_values()[0].title)))
            else:
                tokens.append(("class:answer",
                               ' done ({} selections)'.format(
                                   nbr_selected)))
        else:
            tokens.append(("class:instruction",
                           ' (Use arrow keys to move, '
                           '<space> to select, '
                           '<a> to toggle, '
                           '<i> to invert)'))
        return tokens

    ps = PromptSession(get_prompt_tokens, reserve_space_for_menu=0, **kwargs)

    layout = Layout(HSplit([
        ps.layout.container,
        ConditionalContainer(
            Window(ic),
            filter=~IsDone()
        )
    ]))

    bindings = KeyBindings()

    @bindings.add(Keys.ControlQ, eager=True)
    @bindings.add(Keys.ControlC, eager=True)
    def _(event):
        event.app.exit(exception=KeyboardInterrupt, style='class:aborting')

    @bindings.add(' ', eager=True)
    def toggle(event):
        pointed_choice = ic.get_pointed_at().value
        if pointed_choice in ic.selected_options:
            ic.selected_options.remove(pointed_choice)
        else:
            ic.selected_options.append(pointed_choice)

    @bindings.add('i', eager=True)
    def invert(event):
        inverted_selection = [c.value for c in ic.choices if
                              not isinstance(c, Separator) and
                              c.value not in ic.selected_options and
                              not c.disabled]
        ic.selected_options = inverted_selection

    @bindings.add('a', eager=True)
    def all(event):
        all_selected = True  # all choices have been selected
        for c in ic.choices:
            if(not isinstance(c, Separator) and
                    c.value not in ic.selected_options and
                    not c.disabled):
                # add missing ones
                ic.selected_options.append(c.value)
                all_selected = False
        if all_selected:
            ic.selected_options = []

    @bindings.add(Keys.Down, eager=True)
    def move_cursor_down(event):
        ic.select_next()
        while not ic.is_selection_valid():
            ic.select_next()

    @bindings.add(Keys.Up, eager=True)
    def move_cursor_up(event):
        ic.select_previous()
        while not ic.is_selection_valid():
            ic.select_previous()

    @bindings.add(Keys.ControlM, eager=True)
    def set_answer(event):
        ic.answered = True
        event.app.exit(result=[c.value for c in ic.get_selected_values()])

    @bindings.add(Keys.Any)
    def other(event):
        """Disallow inserting other text. """
        pass

    return Application(
        layout=layout,
        key_bindings=bindings,
        style=merged_style,
        **kwargs
    )



# -*- coding: utf-8 -*-
import re

from prompt_toolkit.validation import Validator, ValidationError

from tests.utils import feed_cli_with_input


def test_input():
    message = 'What is your name'
    text = "bob\r"

    result, cli = feed_cli_with_input('input', message, text)
    assert result == 'bob'


def test_input_validate():
    message = 'What is your name'
    text = "Doe\r"

    result, cli = feed_cli_with_input(
        'input', message, text,
        validate=lambda val: val == 'Doe' or 'is your last name Doe?')
    assert result == 'Doe'


def test_input_validate_with_class():
    class SimpleValidator(Validator):
        def validate(self, document):
            ok = re.match(
                '[01][01][01]',
                document.text)
            if not ok:
                raise ValidationError(message='Binary FTW',
                                      cursor_position=len(document.text))

    message = 'What is your name'
    text = "001\r"

    result, cli = feed_cli_with_input('input', message, text,
                                      validate=SimpleValidator)
    assert result == '001'
